import itertools
import os

import pandas as pd
import yaml

DATA_DIRECTORY = "data"
GI_DATA_LOCATION = os.path.join(DATA_DIRECTORY, "inquirerbasic.xls")
GI_FOLD_WORD_LIST_LOCATION = os.path.join(DATA_DIRECTORY, "GI_fold_words.yaml")
GI_OUT_DATASET_LOCATION = os.path.join(DATA_DIRECTORY, "GI_properties.yaml")

# Load General Inquirer data
df = pd.DataFrame(pd.read_excel(GI_DATA_LOCATION, converters={"Entry": str}))
df.drop(columns="Source", inplace=True)
df.drop(columns="Defined", inplace=True)

# Load vocabulary
with open(GI_FOLD_WORD_LIST_LOCATION) as gi_fold_word_file:
    gi_fold_words = yaml.load(gi_fold_word_file)
vocab = sorted(itertools.chain.from_iterable(gi_fold_words.values()))

# Aggregate data
vocab_counts = {}
property_dataset = {}
for row in df.to_dict("records"):
    concept = row["Entry"].lower()
    # Excel values are column names
    features = [x for x in row.keys() if row[x] == x]
    # No features
    if len(features) == 0:
        continue
    # No POS tags
    if isinstance(row["Othtags"], float):
        continue
    other_tags = set(row["Othtags"].split(" "))
    pos_tags = set()
    if "Noun" in other_tags:
        pos_tags.add("n")
    elif "SUPV" in other_tags:
        pos_tags.add("v")
    elif "Modif" in other_tags:
        pos_tags.add("a")
    elif "LY" in other_tags:
        pos_tags.add("a")
    if len(pos_tags) != 1:
        continue
    if "#" in concept:
        concept = concept.split("#")[0]
    concept = concept + "-" + pos_tags.pop()
    if concept not in vocab:
        continue
    if concept not in property_dataset:
        property_dataset[concept] = {}
        vocab_counts[concept] = 0
    vocab_counts[concept] += 1
    for feature in features:
        if feature not in property_dataset[concept]:
            property_dataset[concept][feature] = 0
        property_dataset[concept][feature] += 1
for concept in property_dataset:
    for feature in property_dataset[concept]:
        property_dataset[concept][feature] /= vocab_counts[concept]

# Write to file
with open(GI_OUT_DATASET_LOCATION, "w") as out_file:
    yaml.dump(property_dataset, out_file, default_flow_style=False)
