import itertools
import os

import pandas as pd
import yaml

DATA_DIRECTORY = "data"
MCRAE_DATA_LOCATION = os.path.join(DATA_DIRECTORY,
                                   "CONCS_FEATS_concstats_brm.xlsx")
MCRAE_FOLD_WORD_LIST_LOCATION = os.path.join(DATA_DIRECTORY,
                                             "McRae_fold_words.yaml")
MCRAE_OUT_DATASET_LOCATION = os.path.join(DATA_DIRECTORY,
                                          "McRae_properties.yaml")

# Load McRae data
df = pd.DataFrame(pd.read_excel(MCRAE_DATA_LOCATION))

# Load vocabulary
with open(MCRAE_FOLD_WORD_LIST_LOCATION) as mcrae_fold_word_file:
    mcrae_fold_words = yaml.load(mcrae_fold_word_file)
vocab = sorted(itertools.chain.from_iterable(mcrae_fold_words.values()))
vocab = set(word[:-2] for word in vocab)

# Aggregate data
property_dataset = {}
for row in df.to_dict("records"):
    concept = row["Concept"]
    if concept not in vocab:
        continue
    concept = concept + "-n"
    if concept not in property_dataset:
        property_dataset[concept] = {}
    feature = row["Feature"]
    freq = int(row["Prod_Freq"])
    property_dataset[concept][feature] = freq / 30

# Write to file
with open(MCRAE_OUT_DATASET_LOCATION, "w") as out_file:
    yaml.dump(property_dataset, out_file, default_flow_style=False)
