import itertools
import os
import zipfile

import pandas as pd
import yaml

DATA_DIRECTORY = "data"
VV_ZIP_LOCATION = os.path.join(DATA_DIRECTORY, "Vinson-BRM-2008a.zip")
VV_UNZIP_LOCATION = os.path.join(DATA_DIRECTORY, "Vinson-BRM-2008a")
VV_FOLD_WORD_LIST_LOCATION = os.path.join(DATA_DIRECTORY, "VV_fold_words.yaml")
VV_OUT_DATASET_LOCATION = os.path.join(DATA_DIRECTORY, "VV_properties.yaml")

# Unzip file
with zipfile.ZipFile(VV_ZIP_LOCATION, "r") as zip_ref:
    zip_ref.extractall(VV_UNZIP_LOCATION)

# Collect concept list (columns of matrix)
gold_concept_file_location = os.path.join(
    VV_UNZIP_LOCATION, "Vinson-BRM-2008", "word_categories.txt"
)
concept_df = pd.read_csv(gold_concept_file_location, sep="\t", header=0)
concept_list = concept_df["word"].tolist()
concept_POS = concept_df["type"].tolist()

# Convert to concept-pos format
gold_vocab = []
for concept, pos in zip(concept_list, concept_POS):
    if pos == "actionV":
        gold_vocab.append(concept.lower() + "-v")
    elif pos == "actionN":
        gold_vocab.append(concept.lower() + "-n")
    elif pos == "object":
        gold_vocab.append(concept.lower() + "-n")
    else:
        print(concept, pos)
        raise ValueError

# Collect Feature list (rows of matrix)
gold_feature_file_location = os.path.join(
    VV_UNZIP_LOCATION, "Vinson-BRM-2008", "feature_list_and_types.txt"
)
feature_df = pd.read_csv(gold_feature_file_location, sep="\t", header=0)
feature_list = feature_df["feature"].tolist()

# Load vocabulary
with open(VV_FOLD_WORD_LIST_LOCATION) as vv_fold_word_file:
    vv_fold_words = yaml.load(vv_fold_word_file)
vocab = sorted(itertools.chain.from_iterable(vv_fold_words.values()))

# Aggregate data from first half of matrix
property_dataset = {}
gold_matrix1_file_location = os.path.join(
    VV_UNZIP_LOCATION, "Vinson-BRM-2008", "feature_weight_matrix_1_256.txt"
)
matrix1 = pd.read_csv(
    gold_matrix1_file_location, sep="\t", header=None
).values.transpose()
for concept_index, feature_index in zip(*matrix1.nonzero()):
    print(concept_index, feature_index)
    concept = gold_vocab[concept_index]
    if concept not in vocab:
        continue
    if concept not in property_dataset:
        property_dataset[concept] = {}
    feature = feature_list[feature_index]
    value = matrix1[concept_index, feature_index]
    property_dataset[concept][feature] = float(value) / 20

# Aggregate data from second half of matrix
gold_matrix2_file_location = os.path.join(
    VV_UNZIP_LOCATION, "Vinson-BRM-2008", "feature_weight_matrix_257_456.txt"
)
matrix2 = pd.read_csv(
    gold_matrix2_file_location, sep="\t", header=None
).values.transpose()
for concept_index, feature_index in zip(*matrix2.nonzero()):
    concept = gold_vocab[concept_index + 256]
    if concept not in vocab:
        continue
    if concept not in property_dataset:
        property_dataset[concept] = {}
    feature = feature_list[feature_index]
    value = matrix2[concept_index, feature_index]
    property_dataset[concept][feature] = float(value) / 20

# Write to file
with open(VV_OUT_DATASET_LOCATION, "w") as out_file:
    yaml.dump(property_dataset, out_file, default_flow_style=False)
