import itertools
import os

import nltk
import yaml
from nltk.corpus import wordnet as wn

# Download WordNet
nltk.download("wordnet")

DATA_DIRECTORY = "data"
WN_FOLD_WORD_LIST_LOCATION = os.path.join(DATA_DIRECTORY, "WN_fold_words.yaml")
WN_OUT_DATASET_LOCATION = os.path.join(DATA_DIRECTORY, "WN_properties.yaml")

# Load vocabulary
with open(WN_FOLD_WORD_LIST_LOCATION) as wn_fold_word_file:
    wn_fold_words = yaml.load(wn_fold_word_file)
vocab = sorted(itertools.chain.from_iterable(wn_fold_words.values()))

# Aggregate data
vocab_counts = {}
property_dataset = {}
for concept in vocab:
    lemma = "-".join(concept.split("-")[:-1])
    pos = concept.split("-")[-1]
    # Get lemmas for word + pos
    lemma_set = wn.lemmas(lemma, pos)
    for lemma in lemma_set:
        # Frequency of lemma in WordNet
        count = lemma.count()
        if count > 0:
            synset = lemma.synset()
            # Get all hypernyms (not just directly above) of lemma's synset
            features = set(synset.closure(lambda s: s.hypernyms()))
            features = map(lambda x: x.name(), features)
            for feature in features:
                if concept not in property_dataset:
                    property_dataset[concept] = {}
                if feature not in property_dataset[concept]:
                    property_dataset[concept][feature] = 0
                property_dataset[concept][feature] += count
            if concept not in vocab_counts:
                vocab_counts[concept] = 0
            vocab_counts[concept] += count

for concept in property_dataset:
    for feature in property_dataset[concept]:
        property_dataset[concept][feature] /= vocab_counts[concept]

# Write to file
with open(WN_OUT_DATASET_LOCATION, "w") as out_file:
    yaml.dump(property_dataset, out_file, default_flow_style=False)
